@@ -123,7 +123,7 @@ int frame_size; // in bytes for interleaved stereo
 int alsa_device_initialised; // boolean to ensure the initialisation is only
                              // done once
 
-enum yndk_type precision_delay_available_status =
+yndk_type precision_delay_available_status =
     YNDK_DONT_KNOW; // initially, we don't know if the device can do precision delay
 
 snd_pcm_t *alsa_handle = NULL;
@@ -159,14 +159,14 @@ int volume_based_mute_is_active =
 snd_pcm_sframes_t (*alsa_pcm_write)(snd_pcm_t *, const void *, snd_pcm_uframes_t) = snd_pcm_writei;
 
 int precision_delay_and_status(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
-                               enum yndk_type *using_update_timestamps);
+                               yndk_type *using_update_timestamps);
 int standard_delay_and_status(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
-                              enum yndk_type *using_update_timestamps);
+                              yndk_type *using_update_timestamps);
 
 // use this to allow the use of standard or precision delay calculations, with standard the, uh,
 // standard.
 int (*delay_and_status)(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
-                        enum yndk_type *using_update_timestamps) = standard_delay_and_status;
+                        yndk_type *using_update_timestamps) = standard_delay_and_status;
 
 // this will return true if the DAC can return precision delay information and false if not
 // if it is not yet known, it will test the output device to find out
@@ -210,13 +210,10 @@ int precision_delay_available() {
           generate_zero_frames(silence, frames_of_silence, config.output_format,
                                use_dither, // i.e. with dither
                                dither_random_number_store);
-      // debug(1,"Play %d frames of silence with most_recent_write_time of
-      // %" PRIx64 ".",
-      //    frames_of_silence,most_recent_write_time);
       do_play(silence, frames_of_silence);
       pthread_cleanup_pop(1);
       // now we can get the delay, and we'll note if it uses update timestamps
-      enum yndk_type uses_update_timestamps;
+      yndk_type uses_update_timestamps;
       snd_pcm_state_t state;
       snd_pcm_sframes_t delay;
       int ret = precision_delay_and_status(&state, &delay, &uses_update_timestamps);
@@ -255,8 +252,6 @@ static uint64_t frames_played_at_measurement_start_time;
 static uint64_t measurement_time;
 static uint64_t frames_played_at_measurement_time;
 
-volatile uint64_t most_recent_write_time;
-
 static uint64_t frames_sent_for_playing;
 static uint64_t frame_index;
 static int measurement_data_is_valid;
@@ -342,8 +337,9 @@ void actual_close_alsa_device() {
   if (alsa_handle) {
     int derr;
     if ((derr = snd_pcm_hw_free(alsa_handle)))
-      debug(1, "Error %d (\"%s\") freeing the output device hardware while "
-               "closing it.",
+      debug(1,
+            "Error %d (\"%s\") freeing the output device hardware while "
+            "closing it.",
             derr, snd_strerror(derr));
 
     if ((derr = snd_pcm_close(alsa_handle)))
@@ -358,7 +354,10 @@ void actual_close_alsa_device() {
 // The lowest rate that the DAC is capable of is chosen.
 
 unsigned int auto_speed_output_rates[] = {
-    44100, 88200, 176400, 352800,
+    44100,
+    88200,
+    176400,
+    352800,
 };
 
 // This array is of all the formats known to Shairport Sync, in order of the SPS_FORMAT definitions,
@@ -392,7 +391,7 @@ format_record fr[] = {
 // be added at the lowest possible level.
 // Hence, selecting the greatest bit depth is always either beneficial or neutral.
 
-enum sps_format_t auto_format_check_sequence[] = {
+sps_format_t auto_format_check_sequence[] = {
     SPS_FORMAT_S32,    SPS_FORMAT_S32_LE,  SPS_FORMAT_S32_BE,  SPS_FORMAT_S24, SPS_FORMAT_S24_LE,
     SPS_FORMAT_S24_BE, SPS_FORMAT_S24_3LE, SPS_FORMAT_S24_3BE, SPS_FORMAT_S16, SPS_FORMAT_S16_LE,
     SPS_FORMAT_S16_BE, SPS_FORMAT_S8,      SPS_FORMAT_U8,
@@ -508,12 +507,12 @@ int actual_open_alsa_device(int do_auto_setup) {
     }
   } else { // auto format
     int number_of_formats_to_try;
-    enum sps_format_t *formats;
+    sps_format_t *formats;
     formats = auto_format_check_sequence;
     number_of_formats_to_try = sizeof(auto_format_check_sequence) / sizeof(sps_format_t);
     int i = 0;
     int format_found = 0;
-    enum sps_format_t trial_format = SPS_FORMAT_UNKNOWN;
+    sps_format_t trial_format = SPS_FORMAT_UNKNOWN;
     while ((i < number_of_formats_to_try) && (format_found == 0)) {
       trial_format = formats[i];
       sf = fr[trial_format].alsa_code;
@@ -707,9 +706,10 @@ int actual_open_alsa_device(int do_auto_setup) {
           buffer_size);
     }
     */
-    debug(1, "The alsa buffer is smaller (%lu bytes) than the desired backend "
-             "buffer "
-             "length (%ld) you have chosen.",
+    debug(1,
+          "The alsa buffer is smaller (%lu bytes) than the desired backend "
+          "buffer "
+          "length (%ld) you have chosen.",
           actual_buffer_length, config.audio_backend_buffer_desired_length);
   }
 
@@ -869,120 +869,117 @@ int prepare_mixer() {
   // do any alsa device initialisation (general case)
   // at present, this is only needed if a hardware mixer is being used
   // if there's a hardware mixer, it needs to be initialised before use
-    if (alsa_mix_ctrl == NULL) {
-      audio_alsa.volume = NULL;
-      audio_alsa.parameters = NULL;
-      audio_alsa.mute = NULL;
-    } else {
-      debug(2, "alsa: hardware mixer prepare");
-      int oldState;
-      pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &oldState); // make this un-cancellable
+  if (alsa_mix_ctrl == NULL) {
+    audio_alsa.volume = NULL;
+    audio_alsa.parameters = NULL;
+    audio_alsa.mute = NULL;
+  } else {
+    debug(2, "alsa: hardware mixer prepare");
+    int oldState;
+    pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &oldState); // make this un-cancellable
 
-      if (alsa_mix_dev == NULL)
-        alsa_mix_dev = alsa_out_dev;
+    if (alsa_mix_dev == NULL)
+      alsa_mix_dev = alsa_out_dev;
 
-      // Now, start trying to initialise the alsa device with the settings
-      // obtained
-      pthread_cleanup_debug_mutex_lock(&alsa_mixer_mutex, 1000, 1);
-      if (open_mixer() == 1) {
-        if (snd_mixer_selem_get_playback_volume_range(alsa_mix_elem, &alsa_mix_minv,
-                                                      &alsa_mix_maxv) < 0)
-          debug(1, "Can't read mixer's [linear] min and max volumes.");
-        else {
-          if (snd_mixer_selem_get_playback_dB_range(alsa_mix_elem, &alsa_mix_mindb,
-                                                    &alsa_mix_maxdb) == 0) {
+    // Now, start trying to initialise the alsa device with the settings
+    // obtained
+    pthread_cleanup_debug_mutex_lock(&alsa_mixer_mutex, 1000, 1);
+    if (open_mixer() == 1) {
+      if (snd_mixer_selem_get_playback_volume_range(alsa_mix_elem, &alsa_mix_minv, &alsa_mix_maxv) <
+          0)
+        debug(1, "Can't read mixer's [linear] min and max volumes.");
+      else {
+        if (snd_mixer_selem_get_playback_dB_range(alsa_mix_elem, &alsa_mix_mindb,
+                                                  &alsa_mix_maxdb) == 0) {
 
-            audio_alsa.volume = &volume;         // insert the volume function now we
-                                                 // know it can do dB stuff
-            audio_alsa.parameters = &parameters; // likewise the parameters stuff
-            if (alsa_mix_mindb == SND_CTL_TLV_DB_GAIN_MUTE) {
-              // For instance, the Raspberry Pi does this
-              debug(1, "Lowest dB value is a mute");
-              mixer_volume_setting_gives_mute = 1;
-              alsa_mix_mute = SND_CTL_TLV_DB_GAIN_MUTE; // this may not be
-                                                        // necessary -- it's
-                                                        // always
-              // going to be SND_CTL_TLV_DB_GAIN_MUTE, right?
-              // debug(1, "Try minimum volume + 1 as lowest true attenuation
-              // value");
-              if (snd_mixer_selem_ask_playback_vol_dB(alsa_mix_elem, alsa_mix_minv + 1,
-                                                      &alsa_mix_mindb) != 0)
-                debug(1, "Can't get dB value corresponding to a minimum volume "
-                         "+ 1.");
-            }
-            debug(3, "Hardware mixer has dB volume from %f to %f.", (1.0 * alsa_mix_mindb) / 100.0,
-                  (1.0 * alsa_mix_maxdb) / 100.0);
-          } else {
-            // use the linear scale and do the db conversion ourselves
-            warn("The hardware mixer specified -- \"%s\" -- does not have "
-                 "a dB volume scale.",
-                 alsa_mix_ctrl);
-
-            if (snd_ctl_open(&ctl, alsa_mix_dev, 0) < 0) {
-              warn("Cannot open control \"%s\"", alsa_mix_dev);
-              response = -1;
-            }
-            if (snd_ctl_elem_id_malloc(&elem_id) < 0) {
-              debug(1, "Cannot allocate memory for control \"%s\"", alsa_mix_dev);
-              elem_id = NULL;
-              response = -2;
-            } else {
-              snd_ctl_elem_id_set_interface(elem_id, SND_CTL_ELEM_IFACE_MIXER);
-              snd_ctl_elem_id_set_name(elem_id, alsa_mix_ctrl);
-
-              if (snd_ctl_get_dB_range(ctl, elem_id, &alsa_mix_mindb, &alsa_mix_maxdb) == 0) {
-                debug(1, "alsa: hardware mixer \"%s\" selected, with dB volume "
-                         "from %f to %f.",
-                      alsa_mix_ctrl, (1.0 * alsa_mix_mindb) / 100.0,
-                      (1.0 * alsa_mix_maxdb) / 100.0);
-                has_softvol = 1;
-                audio_alsa.volume = &volume;         // insert the volume function now
-                                                     // we know it can do dB stuff
-                audio_alsa.parameters = &parameters; // likewise the parameters stuff
-              } else {
-                debug(1, "Cannot get the dB range from the volume control \"%s\"", alsa_mix_ctrl);
-              }
-            }
-            /*
-            debug(1, "Min and max volumes are %d and
-            %d.",alsa_mix_minv,alsa_mix_maxv);
-            alsa_mix_maxdb = 0;
-            if ((alsa_mix_maxv!=0) && (alsa_mix_minv!=0))
-              alsa_mix_mindb =
-            -20*100*(log10(alsa_mix_maxv*1.0)-log10(alsa_mix_minv*1.0));
-            else if (alsa_mix_maxv!=0)
-              alsa_mix_mindb = -20*100*log10(alsa_mix_maxv*1.0);
-            audio_alsa.volume = &linear_volume; // insert the linear volume
-            function
-            audio_alsa.parameters = &parameters; // likewise the parameters
-            stuff
-            debug(1,"Max and min dB calculated are %d and
-            %d.",alsa_mix_maxdb,alsa_mix_mindb);
-            */
+          audio_alsa.volume = &volume;         // insert the volume function now we
+                                               // know it can do dB stuff
+          audio_alsa.parameters = &parameters; // likewise the parameters stuff
+          if (alsa_mix_mindb == SND_CTL_TLV_DB_GAIN_MUTE) {
+            // For instance, the Raspberry Pi does this
+            debug(1, "Lowest dB value is a mute");
+            mixer_volume_setting_gives_mute = 1;
+            alsa_mix_mute = SND_CTL_TLV_DB_GAIN_MUTE; // this may not be
+                                                      // necessary -- it's
+                                                      // always
+            // going to be SND_CTL_TLV_DB_GAIN_MUTE, right?
+            // debug(1, "Try minimum volume + 1 as lowest true attenuation
+            // value");
+            if (snd_mixer_selem_ask_playback_vol_dB(alsa_mix_elem, alsa_mix_minv + 1,
+                                                    &alsa_mix_mindb) != 0)
+              debug(1, "Can't get dB value corresponding to a minimum volume "
+                       "+ 1.");
           }
-        }
-        if (((config.alsa_use_hardware_mute == 1) &&
-             (snd_mixer_selem_has_playback_switch(alsa_mix_elem))) ||
-            mixer_volume_setting_gives_mute) {
-          audio_alsa.mute = &mute; // insert the mute function now we know it
-                                   // can do muting stuff
-          // debug(1, "Has mixer and mute ability we will use.");
+          debug(3, "Hardware mixer has dB volume from %f to %f.", (1.0 * alsa_mix_mindb) / 100.0,
+                (1.0 * alsa_mix_maxdb) / 100.0);
         } else {
-          // debug(1, "Has mixer but not using hardware mute.");
+          // use the linear scale and do the db conversion ourselves
+          warn("The hardware mixer specified -- \"%s\" -- does not have "
+               "a dB volume scale.",
+               alsa_mix_ctrl);
+
+          if (snd_ctl_open(&ctl, alsa_mix_dev, 0) < 0) {
+            warn("Cannot open control \"%s\"", alsa_mix_dev);
+            response = -1;
+          }
+          if (snd_ctl_elem_id_malloc(&elem_id) < 0) {
+            debug(1, "Cannot allocate memory for control \"%s\"", alsa_mix_dev);
+            elem_id = NULL;
+            response = -2;
+          } else {
+            snd_ctl_elem_id_set_interface(elem_id, SND_CTL_ELEM_IFACE_MIXER);
+            snd_ctl_elem_id_set_name(elem_id, alsa_mix_ctrl);
+
+            if (snd_ctl_get_dB_range(ctl, elem_id, &alsa_mix_mindb, &alsa_mix_maxdb) == 0) {
+              debug(1,
+                    "alsa: hardware mixer \"%s\" selected, with dB volume "
+                    "from %f to %f.",
+                    alsa_mix_ctrl, (1.0 * alsa_mix_mindb) / 100.0, (1.0 * alsa_mix_maxdb) / 100.0);
+              has_softvol = 1;
+              audio_alsa.volume = &volume;         // insert the volume function now
+                                                   // we know it can do dB stuff
+              audio_alsa.parameters = &parameters; // likewise the parameters stuff
+            } else {
+              debug(1, "Cannot get the dB range from the volume control \"%s\"", alsa_mix_ctrl);
+            }
+          }
+          /*
+          debug(1, "Min and max volumes are %d and
+          %d.",alsa_mix_minv,alsa_mix_maxv);
+          alsa_mix_maxdb = 0;
+          if ((alsa_mix_maxv!=0) && (alsa_mix_minv!=0))
+            alsa_mix_mindb =
+          -20*100*(log10(alsa_mix_maxv*1.0)-log10(alsa_mix_minv*1.0));
+          else if (alsa_mix_maxv!=0)
+            alsa_mix_mindb = -20*100*log10(alsa_mix_maxv*1.0);
+          audio_alsa.volume = &linear_volume; // insert the linear volume
+          function
+          audio_alsa.parameters = &parameters; // likewise the parameters
+          stuff
+          debug(1,"Max and min dB calculated are %d and
+          %d.",alsa_mix_maxdb,alsa_mix_mindb);
+          */
         }
-        close_mixer();
       }
-      debug_mutex_unlock(&alsa_mixer_mutex, 3); // release the mutex
-      pthread_cleanup_pop(0);
-      pthread_setcancelstate(oldState, NULL);
+      if (((config.alsa_use_hardware_mute == 1) &&
+           (snd_mixer_selem_has_playback_switch(alsa_mix_elem))) ||
+          mixer_volume_setting_gives_mute) {
+        audio_alsa.mute = &mute; // insert the mute function now we know it
+                                 // can do muting stuff
+        // debug(1, "Has mixer and mute ability we will use.");
+      } else {
+        // debug(1, "Has mixer but not using hardware mute.");
+      }
+      close_mixer();
     }
+    debug_mutex_unlock(&alsa_mixer_mutex, 3); // release the mutex
+    pthread_cleanup_pop(0);
+    pthread_setcancelstate(oldState, NULL);
+  }
   return response;
 }
 
-int alsa_device_init() {
-  return prepare_mixer();
-}
-
+int alsa_device_init() { return prepare_mixer(); }
 
 static int init(int argc, char **argv) {
   // for debugging
@@ -1140,8 +1137,10 @@ static int init(int argc, char **argv) {
              "\"S16\", \"S24\", \"S24_LE\", \"S24_BE\", "
              "\"S24_3LE\", \"S24_3BE\" or "
              "\"S32\", \"S32_LE\", \"S32_BE\". It remains set to \"%s\".",
-             str, config.output_format_auto_requested == 1 ? "auto" : sps_format_description_string(
-                                                                          config.output_format));
+             str,
+             config.output_format_auto_requested == 1
+                 ? "auto"
+                 : sps_format_description_string(config.output_format));
       }
     }
 
@@ -1291,9 +1290,9 @@ static int init(int argc, char **argv) {
         warn("Invalid use_precision_timing option choice \"%s\". It should be "
              "\"yes\", \"auto\" or \"no\". "
              "It remains set to \"%s\".",
-             config.use_precision_timing == YNA_NO ? "no" : config.use_precision_timing == YNA_AUTO
-                                                                ? "auto"
-                                                                : "yes");
+             config.use_precision_timing == YNA_NO
+                 ? "no"
+                 : config.use_precision_timing == YNA_AUTO ? "auto" : "yes");
       }
     }
 
@@ -1350,7 +1349,6 @@ static int init(int argc, char **argv) {
   // length of the queue
   // if the queue gets too short, stuff it with silence
 
-  most_recent_write_time = 0; // could be used by the alsa_buffer_monitor_thread_code
   pthread_create(&alsa_buffer_monitor_thread, NULL, &alsa_buffer_monitor_thread_code, NULL);
 
   return response;
@@ -1422,7 +1420,7 @@ static void start(__attribute__((unused)) int i_sample_rate,
 }
 
 int standard_delay_and_status(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
-                              enum yndk_type *using_update_timestamps) {
+                              yndk_type *using_update_timestamps) {
   int ret = 0;
   if (using_update_timestamps)
     *using_update_timestamps = YNDK_NO;
@@ -1444,7 +1442,7 @@ int standard_delay_and_status(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
 }
 
 int precision_delay_and_status(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
-                               enum yndk_type *using_update_timestamps) {
+                               yndk_type *using_update_timestamps) {
   snd_pcm_status_t *alsa_snd_pcm_status;
   snd_pcm_status_alloca(&alsa_snd_pcm_status);
 
@@ -1525,14 +1523,16 @@ int precision_delay_and_status(snd_pcm_state_t *state, snd_pcm_sframes_t *delay,
 
           if (((update_timestamp_ns - stall_monitor_start_time) > stall_monitor_error_threshold) ||
               ((time_now_ns - stall_monitor_start_time) > stall_monitor_error_threshold)) {
-            debug(2, "DAC seems to have stalled with time_now_ns: %" PRIX64
-                     ", update_timestamp_ns: %" PRIX64 ", stall_monitor_start_time %" PRIX64
-                     ", stall_monitor_error_threshold %" PRIX64 ".",
+            debug(2,
+                  "DAC seems to have stalled with time_now_ns: %" PRIX64
+                  ", update_timestamp_ns: %" PRIX64 ", stall_monitor_start_time %" PRIX64
+                  ", stall_monitor_error_threshold %" PRIX64 ".",
                   time_now_ns, update_timestamp_ns, stall_monitor_start_time,
                   stall_monitor_error_threshold);
-            debug(2, "DAC seems to have stalled with time_now: %lx,%lx"
-                     ", update_timestamp: %lx,%lx, stall_monitor_start_time %" PRIX64
-                     ", stall_monitor_error_threshold %" PRIX64 ".",
+            debug(2,
+                  "DAC seems to have stalled with time_now: %lx,%lx"
+                  ", update_timestamp: %lx,%lx, stall_monitor_start_time %" PRIX64
+                  ", stall_monitor_error_threshold %" PRIX64 ".",
                   tn.tv_sec, tn.tv_nsec, update_timestamp.tv_sec, update_timestamp.tv_nsec,
                   stall_monitor_start_time, stall_monitor_error_threshold);
             ret = sps_extra_code_output_stalled;
@@ -1605,6 +1605,7 @@ int delay(long *the_delay) {
 }
 
 int get_rate_information(uint64_t *elapsed_time, uint64_t *frames_played) {
+  // elapsed_time is in nanoseconds
   int response = 0; // zero means okay
   if (measurement_data_is_valid) {
     *elapsed_time = measurement_time - measurement_start_time;
@@ -1657,7 +1658,7 @@ int do_play(void *buf, int samples) {
         if ((frame_index == start_measurement_from_this_frame) ||
             ((frame_index > start_measurement_from_this_frame) && (frame_index % 32 == 0))) {
 
-          measurement_time = get_absolute_time_in_fp();
+          measurement_time = get_absolute_time_in_ns();
           frames_played_at_measurement_time = frames_sent_for_playing - my_delay - samples;
 
           if (frame_index == start_measurement_from_this_frame) {
@@ -1697,8 +1698,9 @@ int do_play(void *buf, int samples) {
       }
     }
   } else {
-    debug(1, "alsa: device status returns fault status %d and SND_PCM_STATE_* "
-             "%d  for play.",
+    debug(1,
+          "alsa: device status returns fault status %d and SND_PCM_STATE_* "
+          "%d  for play.",
           ret, state);
     frame_index = 0;
     measurement_data_is_valid = 0;
@@ -1744,6 +1746,7 @@ int do_close() {
     // debug(1,"alsa: do_close() -- closing the output device");
     if ((derr = snd_pcm_drop(alsa_handle)))
       debug(1, "Error %d (\"%s\") dropping output device.", derr, snd_strerror(derr));
+    usleep(5000);
     if ((derr = snd_pcm_hw_free(alsa_handle)))
       debug(1, "Error %d (\"%s\") freeing the output device hardware.", derr, snd_strerror(derr));
 
@@ -1946,7 +1949,7 @@ void *alsa_buffer_monitor_thread_code(__attribute__((unused)) void *arg) {
       alsa_device_init();
       alsa_device_initialised = 1;
     }
-    int sleep_time_ms = (int)(config.disable_standby_mode_silence_scan_interval * 1000);
+    int sleep_time_us = (int)(config.disable_standby_mode_silence_scan_interval * 1000000);
     pthread_cleanup_debug_mutex_lock(&alsa_mutex, 200000, 0);
     // check possible state transitions here
     if ((alsa_backend_state == abm_disconnected) && (config.keep_dac_busy != 0)) {
@@ -1974,65 +1977,60 @@ void *alsa_buffer_monitor_thread_code(__attribute__((unused)) void *arg) {
       int reply;
       long buffer_size = 0;
       snd_pcm_state_t state;
-      uint64_t present_time = get_absolute_time_in_fp();
-      if ((most_recent_write_time == 0) || (present_time > most_recent_write_time)) {
-        reply = delay_and_status(&state, &buffer_size, NULL);
-        if (reply != 0) {
-          buffer_size = 0;
-          char errorstring[1024];
-          strerror_r(-reply, (char *)errorstring, sizeof(errorstring));
-          debug(1, "alsa: alsa_buffer_monitor_thread_code delay error %d: \"%s\".", reply,
-                (char *)errorstring);
-        }
-        long buffer_size_threshold =
-            (long)(config.disable_standby_mode_silence_threshold * config.output_rate);
-        size_t size_of_silence_buffer;
-        if (buffer_size < buffer_size_threshold) {
-          uint64_t sleep_time_in_fp = sleep_time_ms;
-          sleep_time_in_fp = sleep_time_in_fp << 32;
-          sleep_time_in_fp = sleep_time_in_fp / 1000;
-          int frames_of_silence = 1024;
-          size_of_silence_buffer = frames_of_silence * frame_size;
-          void *silence = malloc(size_of_silence_buffer);
-          if (silence == NULL) {
-            warn("disable_standby_mode has been turned off because a memory allocation error "
-                 "occurred.");
-            error_detected = 1;
-          } else {
-            int ret;
-            pthread_cleanup_push(malloc_cleanup, silence);
-            int use_dither = 0;
-            if ((alsa_mix_ctrl == NULL) && (config.ignore_volume_control == 0) &&
-                (config.airplay_volume != 0.0))
-              use_dither = 1;
-            dither_random_number_store =
-                generate_zero_frames(silence, frames_of_silence, config.output_format,
-                                     use_dither, // i.e. with dither
-                                     dither_random_number_store);
-            ret = do_play(silence, frames_of_silence);
-            frame_count++;
-            pthread_cleanup_pop(1); // free malloced buffer
-            if (ret < 0) {
-              error_count++;
-              char errorstring[1024];
-              strerror_r(-ret, (char *)errorstring, sizeof(errorstring));
-              debug(2, "alsa: alsa_buffer_monitor_thread_code error %d (\"%s\") writing %d samples "
-                       "to alsa device -- %d errors in %d trials.",
-                    ret, (char *)errorstring, frames_of_silence, error_count, frame_count);
-              if ((error_count > 40) && (frame_count < 100)) {
-                warn("disable_standby_mode has been turned off because too many underruns "
-                     "occurred. Is Shairport Sync outputting to a virtual device or running in a "
-                     "virtual machine?");
-                error_detected = 1;
-              }
+      reply = delay_and_status(&state, &buffer_size, NULL);
+      if (reply != 0) {
+        buffer_size = 0;
+        char errorstring[1024];
+        strerror_r(-reply, (char *)errorstring, sizeof(errorstring));
+        debug(1, "alsa: alsa_buffer_monitor_thread_code delay error %d: \"%s\".", reply,
+              (char *)errorstring);
+      }
+      long buffer_size_threshold =
+          (long)(config.disable_standby_mode_silence_threshold * config.output_rate);
+      size_t size_of_silence_buffer;
+      if (buffer_size < buffer_size_threshold) {
+        int frames_of_silence = 1024;
+        size_of_silence_buffer = frames_of_silence * frame_size;
+        void *silence = malloc(size_of_silence_buffer);
+        if (silence == NULL) {
+          warn("disable_standby_mode has been turned off because a memory allocation error "
+               "occurred.");
+          error_detected = 1;
+        } else {
+          int ret;
+          pthread_cleanup_push(malloc_cleanup, silence);
+          int use_dither = 0;
+          if ((alsa_mix_ctrl == NULL) && (config.ignore_volume_control == 0) &&
+              (config.airplay_volume != 0.0))
+            use_dither = 1;
+          dither_random_number_store =
+              generate_zero_frames(silence, frames_of_silence, config.output_format,
+                                   use_dither, // i.e. with dither
+                                   dither_random_number_store);
+          ret = do_play(silence, frames_of_silence);
+          frame_count++;
+          pthread_cleanup_pop(1); // free malloced buffer
+          if (ret < 0) {
+            error_count++;
+            char errorstring[1024];
+            strerror_r(-ret, (char *)errorstring, sizeof(errorstring));
+            debug(2,
+                  "alsa: alsa_buffer_monitor_thread_code error %d (\"%s\") writing %d samples "
+                  "to alsa device -- %d errors in %d trials.",
+                  ret, (char *)errorstring, frames_of_silence, error_count, frame_count);
+            if ((error_count > 40) && (frame_count < 100)) {
+              warn("disable_standby_mode has been turned off because too many underruns "
+                   "occurred. Is Shairport Sync outputting to a virtual device or running in a "
+                   "virtual machine?");
+              error_detected = 1;
             }
           }
         }
       }
     }
     debug_mutex_unlock(&alsa_mutex, 0);
-    pthread_cleanup_pop(0);       // release the mutex
-    usleep(sleep_time_ms * 1000); // has a cancellation point in it
+    pthread_cleanup_pop(0); // release the mutex
+    usleep(sleep_time_us);  // has a cancellation point in it
   }
   pthread_exit(NULL);
 }
