@@ -1,6 +1,6 @@
 /*
  * DACP protocol handler. This file is part of Shairport Sync.
- * Copyright (c) Mike Brady 2017 -- 2019
+ * Copyright (c) Mike Brady 2017 -- 2020
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
@@ -49,13 +49,14 @@
 typedef struct {
   int players_connection_thread_index; // the connection thread index when a player thread is
                                        // associated with this, zero otherwise
-  int scan_enable;                     // set to 1 if if sacanning should be considered
+  int scan_enable;                     // set to 1 if scanning should be considered
   char dacp_id[256];                   // the DACP ID string
   uint16_t port;                       // zero if no port discovered
   short connection_family;             // AF_INET6 or AF_INET
+  int always_use_revision_number_1;    // for dealing with forked-daapd;
   uint32_t scope_id;                   // if it's an ipv6 connection, this will be its scope id
   char ip_string[INET6_ADDRSTRLEN];    // the ip string pointing to the client
-  uint32_t active_remote_id;           // send this when you want to send remote control commands
+  char *active_remote_id;              // send this when you want to send remote control commands
   void *port_monitor_private_storage;
 } dacp_server_record;
 
@@ -122,7 +123,7 @@ static const struct http_funcs responseFuncs = {
 // static pthread_mutex_t dacp_server_information_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t dacp_conversation_lock;
 static pthread_mutex_t dacp_server_information_lock;
-static pthread_cond_t dacp_server_information_cv = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t dacp_server_information_cv;
 
 void addrinfo_cleanup(void *arg) {
   // debug(1, "addrinfo cleanup called.");
@@ -150,12 +151,12 @@ void http_cleanup(void *arg) {
 
 int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
   int result;
-	// debug(1,"dacp_send_command: command is: \"%s\".",command);
-	
-	if (dacp_server.port == 0) {
-	  debug(1,"No DACP port specified yet");
-	  result = 490; // no port specified
-	} else {
+  // debug(1,"dacp_send_command: command is: \"%s\".",command);
+
+  if (dacp_server.port == 0) {
+    // debug(3, "No DACP port specified yet");
+    result = 490; // no port specified
+  } else {
 
     // will malloc space for the body or set it to NULL -- the caller should free it.
 
@@ -168,6 +169,7 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
     //  493 Client failed to send a message
     //  492 Argument out of range
     //  491 Client refused connection
+    //  490 No port specified
 
     struct addrinfo hints, *res;
     int sockfd;
@@ -200,7 +202,7 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
       // debug(1,"Error %d \"%s\" at getaddrinfo.",ires,gai_strerror(ires));
       response.code = 498; // Bad Address information for the DACP server
     } else {
-      uint64_t start_time = get_absolute_time_in_fp();
+      uint64_t start_time = get_absolute_time_in_ns();
       pthread_cleanup_push(addrinfo_cleanup, (void *)&res);
       // only do this one at a time -- not sure it is necessary, but better safe than sorry
 
@@ -213,12 +215,15 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
         sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
 
         if (sockfd == -1) {
-          // debug(1, "DACP socket could not be created -- error %d: \"%s\".",errno,strerror(errno));
+          // debug(1, "DACP socket could not be created -- error %d:
+          // \"%s\".",errno,strerror(errno));
           response.code = 497; // Can't establish a socket to the DACP server
         } else {
           pthread_cleanup_push(connect_cleanup, (void *)&sockfd);
           // debug(2, "dacp_send_command: open socket %d.",sockfd);
 
+          // This is for limiting the time to be spent waiting for a response.
+
           struct timeval tv;
           tv.tv_sec = 0;
           tv.tv_usec = 500000;
@@ -239,8 +244,9 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
             // debug(1,"DACP connect succeeded.");
 
             snprintf(message, sizeof(message),
-                     "GET /ctrl-int/1/%s HTTP/1.1\r\nHost: %s:%u\r\nActive-Remote: %u\r\n\r\n",
-                     command, dacp_server.ip_string, dacp_server.port, dacp_server.active_remote_id);
+                     "GET /ctrl-int/1/%s HTTP/1.1\r\nHost: %s:%u\r\nActive-Remote: %s\r\n\r\n",
+                     command, dacp_server.ip_string, dacp_server.port,
+                     dacp_server.active_remote_id);
 
             // Send command
             debug(3, "dacp_send_command: \"%s\".", command);
@@ -289,10 +295,12 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
                     struct linger so_linger;
                     so_linger.l_onoff = 1; // "true"
                     so_linger.l_linger = 0;
-                    int err = setsockopt(sockfd, SOL_SOCKET, SO_LINGER, &so_linger, sizeof so_linger);
+                    int err =
+                        setsockopt(sockfd, SOL_SOCKET, SO_LINGER, &so_linger, sizeof so_linger);
                     if (err)
-                      debug(1,
-                            "Could not set the dacp socket to abort due to a read error on closing.");
+                      debug(
+                          1,
+                          "Could not set the dacp socket to abort due to a read error on closing.");
                   }
 
                   free(response.body);
@@ -343,10 +351,9 @@ int dacp_send_command(const char *command, char **body, ssize_t *bodysize) {
       }
       pthread_cleanup_pop(1); // this should free the addrinfo
       // freeaddrinfo(res);
-      uint64_t et = get_absolute_time_in_fp() - start_time;
-      et = (et * 1000000) >> 32; // microseconds
+      uint64_t et = get_absolute_time_in_ns() - start_time; // this will be in nanoseconds
       debug(3, "dacp_send_command: %f seconds, response code %d, command \"%s\".",
-            (1.0 * et) / 1000000, response.code, command);
+            (1.0 * et) / 1000000000, response.code, command);
     }
     *body = response.body;
     *bodysize = response.size;
@@ -399,10 +406,20 @@ void set_dacp_server_information(rtsp_conn_info *conn) {
     dacp_server.connection_family = conn->connection_ip_family;
     dacp_server.scope_id = conn->self_scope_id;
     strncpy(dacp_server.ip_string, conn->client_ip_string, INET6_ADDRSTRLEN);
-    debug(3, "set_dacp_server_information set IP to \"%s\" and DACP id to \"%s\".",
+    debug(2, "set_dacp_server_information set IP to \"%s\" and DACP id to \"%s\".",
           dacp_server.ip_string, dacp_server.dacp_id);
 
-    mdns_dacp_monitor_set_id(dacp_server.dacp_id);
+    // If the client is forked-daapd, then we always use revision number 1
+    // because otherwise the return read will hang in a "long poll" if there
+    // are no changes.
+    // This is different to other AirPlay clients
+    // which return immediately with a 403 code if there are no changes.
+    dacp_server.always_use_revision_number_1 = 0;
+    char *p = strstr(conn->UserAgent, "forked-daapd");
+    if ((p != 0) &&
+        (p == conn->UserAgent)) { // must exist and be at the start of the UserAgent string
+      dacp_server.always_use_revision_number_1 = 1;
+    }
 
     metadata_hub_modify_prolog();
     int ch = metadata_store.dacp_server_active != dacp_server.scan_enable;
@@ -426,10 +443,12 @@ void set_dacp_server_information(rtsp_conn_info *conn) {
       //      metadata_hub_modify_epilog(ch);
     }
   }
-  dacp_server.active_remote_id = conn->dacp_active_remote; // even if the dacp_id remains the same,
-                                                           // the active remote will change.
-  debug(3, "set_dacp_server_information set active-remote id to %" PRIu32 ".",
-        dacp_server.active_remote_id);
+  if (dacp_server.active_remote_id)
+    free(dacp_server.active_remote_id);
+  dacp_server.active_remote_id =
+      strdup(conn->dacp_active_remote); // even if the dacp_id remains the same,
+                                        // the active remote will change.
+  debug(3, "set_dacp_server_information set active-remote id to %s.", dacp_server.active_remote_id);
   pthread_cond_signal(&dacp_server_information_cv);
   debug_mutex_unlock(&dacp_server_information_lock, 3);
 }
@@ -440,20 +459,24 @@ void dacp_monitor_port_update_callback(char *dacp_id, uint16_t port) {
         "dacp_monitor_port_update_callback with Remote ID \"%s\", target ID \"%s\" and port "
         "number %d.",
         dacp_id, dacp_server.dacp_id, port);
-  if (strcmp(dacp_id, dacp_server.dacp_id) == 0) {
-    dacp_server.port = port;
-    if (port == 0)
-      dacp_server.scan_enable = 0;
-    else {
-      dacp_server.scan_enable = 1;
-      // debug(2, "dacp_monitor_port_update_callback enables scan");
-    }
-    //    metadata_hub_modify_prolog();
-    //    int ch = metadata_store.dacp_server_active != dacp_server.scan_enable;
-    //    metadata_store.dacp_server_active = dacp_server.scan_enable;
-    //    metadata_hub_modify_epilog(ch);
+  if ((dacp_id == NULL) || (dacp_server.dacp_id == NULL)) {
+    warn("dacp_id or dacp_server.dacp_id NULL detected");
   } else {
-    debug(1, "dacp port monitor reporting on an out-of-use remote.");
+    if (strcmp(dacp_id, dacp_server.dacp_id) == 0) {
+      dacp_server.port = port;
+      if (port == 0)
+        dacp_server.scan_enable = 0;
+      else {
+        dacp_server.scan_enable = 1;
+        // debug(2, "dacp_monitor_port_update_callback enables scan");
+      }
+      //    metadata_hub_modify_prolog();
+      //    int ch = metadata_store.dacp_server_active != dacp_server.scan_enable;
+      //    metadata_store.dacp_server_active = dacp_server.scan_enable;
+      //    metadata_hub_modify_epilog(ch);
+    } else {
+      debug(1, "dacp port monitor reporting on an out-of-use remote.");
+    }
   }
   pthread_cond_signal(&dacp_server_information_cv);
   debug_mutex_unlock(&dacp_server_information_lock, 3);
@@ -466,6 +489,7 @@ void dacp_monitor_thread_code_cleanup(__attribute__((unused)) void *arg) {
 
 void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
   int scan_index = 0;
+  int always_use_revision_number_1 = 0;
   // char server_reply[10000];
   // debug(1, "DACP monitor thread started.");
   // wait until we get a valid port number to begin monitoring it
@@ -486,95 +510,138 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
                (metadata_store.advanced_dacp_server_active != 0);
       metadata_store.dacp_server_active = 0;
       metadata_store.advanced_dacp_server_active = 0;
-      debug(2,
-            "setting metadata_store.dacp_server_active and metadata_store.advanced_dacp_server_active to 0 with an update "
+      /*
+      debug(3,
+            "setting metadata_store.dacp_server_active and "
+            "metadata_store.advanced_dacp_server_active to 0 with an update "
             "flag value of %d",
             ch);
+      */
       metadata_hub_modify_epilog(ch);
-      while (dacp_server.scan_enable == 0) {
-        // debug(1, "dacp_monitor_thread_code wait for an event to possible enable scan");
-        pthread_cond_wait(&dacp_server_information_cv, &dacp_server_information_lock);
-        //      debug(1,"dacp_monitor_thread_code wake up.");
+
+      uint64_t time_to_wait_for_wakeup_ns =
+          (uint64_t)(1000000000 * config.missing_port_dacp_scan_interval_seconds);
+
+#ifdef COMPILE_FOR_LINUX_AND_FREEBSD_AND_CYGWIN_AND_OPENBSD
+      uint64_t time_of_wakeup_ns = get_absolute_time_in_ns() + time_to_wait_for_wakeup_ns;
+      uint64_t sec = time_of_wakeup_ns / 1000000000;
+      uint64_t nsec = time_of_wakeup_ns % 1000000000;
+#endif
+#ifdef COMPILE_FOR_OSX
+      uint64_t sec = time_to_wait_for_wakeup_ns / 1000000000;
+      uint64_t nsec = time_to_wait_for_wakeup_ns % 1000000000;
+#endif
+
+      struct timespec time_to_wait;
+      time_to_wait.tv_sec = sec;
+      time_to_wait.tv_nsec = nsec;
+
+      while ((dacp_server.scan_enable == 0) && (result != ETIMEDOUT)) {
+        // debug(1, "dacp_monitor_thread_code wait for an event to possibly enable scan");
+
+#ifdef COMPILE_FOR_LINUX_AND_FREEBSD_AND_CYGWIN_AND_OPENBSD
+        result = pthread_cond_timedwait(&dacp_server_information_cv, &dacp_server_information_lock,
+                                        &time_to_wait); // this is a pthread cancellation point
+        // debug(1, "result is %d, dacp_server.scan_enable is %d, time_to_wait_for_wakeup_ns is %"
+        // PRId64 ".", result, dacp_server.scan_enable, time_to_wait_for_wakeup_ns);
+
+#endif
+#ifdef COMPILE_FOR_OSX
+        result = pthread_cond_timedwait_relative_np(&dacp_server_information_cv,
+                                                    &dacp_server_information_lock, &time_to_wait);
+#endif
+      }
+      if (dacp_server.scan_enable == 1) {
+        bad_result_count = 0;
+        idle_scan_count = 0;
       }
-      // so dacp_server.scan_enable will be true at this point
-      bad_result_count = 0;
-      idle_scan_count = 0;
-      
     }
-    
+
+    always_use_revision_number_1 =
+        dacp_server.always_use_revision_number_1; // set this while access is locked
+
     result = dacp_get_volume(&the_volume); // just want the http code
     pthread_cleanup_pop(1);
 
-    scan_index++;
-    // debug(1,"DACP Scan Result: %d.", result);
-
-    if ((result == 200) || (result == 400)) {
-      bad_result_count = 0;
+    if (result == 490) { // 490 means no port was specified
+      if (((char *)dacp_server.dacp_id != NULL) && (strlen(dacp_server.dacp_id) != 0)) {
+        // debug(1,"mdns_dacp_monitor_set_id");
+        mdns_dacp_monitor_set_id(dacp_server.dacp_id);
+      }
     } else {
-      if (bad_result_count < config.scan_max_bad_response_count) // limit to some reasonable value
-        bad_result_count++;
-    }
-        
-    // here, do the debouncing calculations to see
-    // if the dacp server  and the
-    // advanced dacp server are available 
-    
-    // -1 means we don't know because some bad statuses have been reported
-    // 0 means definitely no
-    // +1 means definitely yes
-    
-    int dacp_server_status_now = -1;
-    int advanced_dacp_server_status_now = -1;
-    
-    if (bad_result_count == 0) {    
-      dacp_server_status_now = 1;
-      if (result == 200)
-        advanced_dacp_server_status_now = 1;
-      else if (result == 400)
+      scan_index++;
+      // debug(1,"DACP Scan Result: %d.", result);
+
+      if ((result == 200) || (result == 400)) {
+        bad_result_count = 0;
+      } else {
+        if (bad_result_count < config.scan_max_bad_response_count) // limit to some reasonable value
+          bad_result_count++;
+      }
+
+      // here, do the debouncing calculations to see
+      // if the dacp server  and the
+      // advanced dacp server are available
+
+      // -1 means we don't know because some bad statuses have been reported
+      // 0 means definitely no
+      // +1 means definitely yes
+
+      int dacp_server_status_now = -1;
+      int advanced_dacp_server_status_now = -1;
+
+      if (bad_result_count == 0) {
+        dacp_server_status_now = 1;
+        if (result == 200)
+          advanced_dacp_server_status_now = 1;
+        else if (result == 400)
+          advanced_dacp_server_status_now = 0;
+      } else if (bad_result_count ==
+                 config.scan_max_bad_response_count) { // if a sequence of bad return codes occurs,
+                                                       // then it's gone
+        dacp_server_status_now = 0;
         advanced_dacp_server_status_now = 0;
-    } else if (bad_result_count == config.scan_max_bad_response_count) { // if a sequence of bad return codes occurs, then it's gone
-      dacp_server_status_now = 0;
-      advanced_dacp_server_status_now = 0;
-    }
-
-    if (metadata_store.player_thread_active == 0)
-      idle_scan_count++;
-    else
-      idle_scan_count = 0;
-
-    debug(3, "Scan Result: %d, Bad Scan Count: %d, Idle Scan Count: %d.", result, bad_result_count,
-          idle_scan_count);
-
-/* not used
-// decide if idle for too long
-    if  (idle_scan_count == config.scan_max_inactive_count) {
-      debug(2, "DACP server status scanning stopped.");
-      dacp_server.scan_enable = 0;
-    }
-*/
- 
-    int update_needed = 0;
-    metadata_hub_modify_prolog();
-    if (dacp_server_status_now != -1) { // if dacp_server_status_now is actually known...
-      if (metadata_store.dacp_server_active != dacp_server_status_now) {
-        debug(2, "metadata_store.dacp_server_active set to %d.", dacp_server_status_now);
-        metadata_store.dacp_server_active = dacp_server_status_now;
-        update_needed = 1;
       }
-    }
-    if (advanced_dacp_server_status_now != -1) { // if advanced_dacp_server_status_now is actually known...
-      if (metadata_store.advanced_dacp_server_active != advanced_dacp_server_status_now) {
-        debug(2, "metadata_store.advanced_dacp_server_active set to %d.", dacp_server_status_now);
-        metadata_store.advanced_dacp_server_active = advanced_dacp_server_status_now;
-        update_needed = 1;
-      }
-    }
 
-    metadata_hub_modify_epilog(update_needed);
-    
-    // pthread_mutex_unlock(&dacp_server_information_lock);
-    // debug(1, "DACP Server ID \"%u\" at \"%s:%u\", scan %d.", dacp_server.active_remote_id,
-    //      dacp_server.ip_string, dacp_server.port, scan_index);
+      if (metadata_store.player_thread_active == 0)
+        idle_scan_count++;
+      else
+        idle_scan_count = 0;
+
+      debug(3, "Scan Result: %d, Bad Scan Count: %d, Idle Scan Count: %d.", result,
+            bad_result_count, idle_scan_count);
+
+      /* not used
+      // decide if idle for too long
+          if  (idle_scan_count == config.scan_max_inactive_count) {
+            debug(2, "DACP server status scanning stopped.");
+            dacp_server.scan_enable = 0;
+          }
+      */
+
+      int update_needed = 0;
+      metadata_hub_modify_prolog();
+      if (dacp_server_status_now != -1) { // if dacp_server_status_now is actually known...
+        if (metadata_store.dacp_server_active != dacp_server_status_now) {
+          debug(2, "metadata_store.dacp_server_active set to %d.", dacp_server_status_now);
+          metadata_store.dacp_server_active = dacp_server_status_now;
+          update_needed = 1;
+        }
+      }
+      if (advanced_dacp_server_status_now !=
+          -1) { // if advanced_dacp_server_status_now is actually known...
+        if (metadata_store.advanced_dacp_server_active != advanced_dacp_server_status_now) {
+          debug(2, "metadata_store.advanced_dacp_server_active set to %d.", dacp_server_status_now);
+          metadata_store.advanced_dacp_server_active = advanced_dacp_server_status_now;
+          update_needed = 1;
+        }
+      }
+
+      metadata_hub_modify_epilog(update_needed);
+
+      // pthread_mutex_unlock(&dacp_server_information_lock);
+      // debug(1, "DACP Server ID \"%u\" at \"%s:%u\", scan %d.", dacp_server.active_remote_id,
+      //      dacp_server.ip_string, dacp_server.port, scan_index);
 
       if (result == 200) {
         metadata_hub_modify_prolog();
@@ -587,11 +654,15 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
         char *response = NULL;
         int32_t item_size;
         char command[1024] = "";
+        if (always_use_revision_number_1 != 0) // for forked-daapd
+          revision_number = 1;
         snprintf(command, sizeof(command) - 1, "playstatusupdate?revision-number=%d",
                  revision_number);
         // debug(1,"dacp_monitor_thread_code: command: \"%s\"",command);
         result = dacp_send_command(command, &response, &le);
         // debug(1,"Response to \"%s\" is %d.",command,result);
+        // remember: unless the revision_number you pass in is 1,
+        // response will be 200 only if there's something new to report.
         if (result == 200) {
           // if (0) {
           char *sp = response;
@@ -622,7 +693,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
                 case 'cmsr': // revision number
                   t = sp - item_size;
                   revision_number = ntohl(*(uint32_t *)(t));
-                  // debug(1,"    Serial Number: %d", revision_number);
+                  // debug(1,"New revision number received: %d", revision_number);
                   break;
                 case 'caps': // play status
                   t = sp - item_size;
@@ -701,30 +772,32 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
                   break;
                 case 'cann': // track name
                   debug(2, "DACP Track Name seen");
-                  if (string_update_with_size(&metadata_store.track_name, &metadata_store.track_name_changed,
-                                     sp - item_size, item_size)) {
+                  if (string_update_with_size(&metadata_store.track_name,
+                                              &metadata_store.track_name_changed, sp - item_size,
+                                              item_size)) {
                     debug(2, "DACP Track Name set to: \"%s\"", metadata_store.track_name);
                   }
                   break;
                 case 'cana': // artist name
                   debug(2, "DACP Artist Name seen");
                   if (string_update_with_size(&metadata_store.artist_name,
-                                     &metadata_store.artist_name_changed, sp - item_size,
-                                     item_size)) {
+                                              &metadata_store.artist_name_changed, sp - item_size,
+                                              item_size)) {
                     debug(2, "DACP Artist Name set to: \"%s\"", metadata_store.artist_name);
                   }
                   break;
                 case 'canl': // album name
                   debug(2, "DACP Album Name seen");
-                  if (string_update_with_size(&metadata_store.album_name, &metadata_store.album_name_changed,
-                                     sp - item_size, item_size)) {
+                  if (string_update_with_size(&metadata_store.album_name,
+                                              &metadata_store.album_name_changed, sp - item_size,
+                                              item_size)) {
                     debug(2, "DACP Album Name set to: \"%s\"", metadata_store.album_name);
                   }
                   break;
                 case 'cang': // genre
                   debug(2, "DACP Genre seen");
                   if (string_update_with_size(&metadata_store.genre, &metadata_store.genre_changed,
-                                     sp - item_size, item_size)) {
+                                              sp - item_size, item_size)) {
                     debug(2, "DACP Genre set to: \"%s\"", metadata_store.genre);
                   }
                   break;
@@ -809,7 +882,8 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
               }
 
               // finished possibly writing to the metadata hub
-              metadata_hub_modify_epilog(1); // should really see if this can be made responsive to changes
+              metadata_hub_modify_epilog(
+                  1); // should really see if this can be made responsive to changes
             } else {
               debug(1, "Status Update not found.\n");
             }
@@ -849,6 +923,7 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
         sleep(config.scan_interval_when_active);
       else
         sleep(config.scan_interval_when_inactive);
+    }
   }
   debug(1, "DACP monitor thread exiting -- should never happen.");
   pthread_exit(NULL);
@@ -856,6 +931,20 @@ void *dacp_monitor_thread_code(__attribute__((unused)) void *na) {
 
 void dacp_monitor_start() {
   int rc;
+
+#ifdef COMPILE_FOR_LINUX_AND_FREEBSD_AND_CYGWIN_AND_OPENBSD
+  pthread_condattr_t attr;
+  pthread_condattr_init(&attr);
+  pthread_condattr_setclock(&attr, CLOCK_MONOTONIC); // can't do this in OS X, and don't need it.
+  rc = pthread_cond_init(&dacp_server_information_cv, &attr);
+  if (rc)
+    debug(1, "Error initialising the DACP Server Information Condition Variable");
+  pthread_condattr_destroy(&attr);
+#endif
+#ifdef COMPILE_FOR_OSX
+  rc = pthread_cond_init(&dacp_server_information_cv, NULL);
+#endif
+
   pthread_mutexattr_t mta;
 
   rc = pthread_mutexattr_init(&mta);
@@ -914,6 +1003,12 @@ void dacp_monitor_stop() {
     pthread_mutex_destroy(&dacp_server_information_lock);
     debug(3, "DACP Conversation Lock Mutex Destroyed");
     pthread_mutex_destroy(&dacp_conversation_lock);
+    pthread_cond_destroy(&dacp_server_information_cv);
+    debug(3, "DACP Server Information Condition Variable destroyed.");
+    if (dacp_server.active_remote_id) {
+      free(dacp_server.active_remote_id);
+      dacp_server.active_remote_id = NULL;
+    }
   }
 }
 
@@ -981,7 +1076,7 @@ int dacp_set_include_speaker_volume(int64_t machine_number, int32_t vo) {
   snprintf(message, sizeof(message),
            "setproperty?include-speaker-id=%" PRId64 "&dmcp.volume=%" PRId32 "", machine_number,
            vo);
-  debug(1, "sending \"%s\"", message);
+  debug(2, "sending \"%s\"", message);
   return send_simple_dacp_command(message);
   // should return 204
 }
@@ -991,7 +1086,7 @@ int dacp_set_speaker_volume(int64_t machine_number, int32_t vo) {
   memset(message, 0, sizeof(message));
   snprintf(message, sizeof(message), "setproperty?speaker-id=%" PRId64 "&dmcp.volume=%" PRId32 "",
            machine_number, vo);
-  debug(1, "sending \"%s\"", message);
+  debug(2, "sending \"%s\"", message);
   return send_simple_dacp_command(message);
   // should return 204
 }
@@ -1164,9 +1259,9 @@ int dacp_get_volume(int32_t *the_actual_volume) {
       // metadata_store.speaker_volume = actual_volume;
       // metadata_hub_modify_epilog(1);
     } else {
-      debug(1, "Unexpected return code %d from dacp_get_speaker_list.", http_response);
+      debug(2, "Unexpected return code %d from dacp_get_speaker_list.", http_response);
     }
-  } else {
+  } else if ((http_response != 400) && (http_response != 490)) {
     debug(3, "Unexpected return code %d from dacp_get_client_volume.", http_response);
   }
   if (the_actual_volume) {
